import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'custom_route.dart';
import 'src/error_page.dart';
import 'src/route_type.dart';
import 'package:meta/meta.dart';

@protected
abstract class Router {
  static Route onGenerateRoute(RouteSettings settings) {
    Route route;
    _routes.forEach((value) {
      if (value.routeName == settings.name) {
        switch (value.routeType) {
          case RouteType.MATERIAL:
            route = _byMaterial(value);
            break;
          case RouteType.CUPERTINO:
            route = _byCupertino(value);
            break;
          case RouteType.CUSTOM:
            if (value.transitions != null) route = _withTransition(value);

            break;
        }
      }
    });
    if (route == null) {
      route = PageRouteBuilder(
          pageBuilder: (_, __, ___) => ErrorPage(
                value: settings.name,
              ));
    }
    return route;
  }

  static MaterialPageRoute _byMaterial(CustomRoute route) {
    return MaterialPageRoute(
        builder: route.builder,
        settings: route.settings,
        maintainState: route.maintainState,
        fullscreenDialog: route.fullscreenDialog);
  }

  static CupertinoPageRoute _byCupertino(CustomRoute route) {
    return CupertinoPageRoute(
        builder: route.builder,
        settings: route.settings,
        title: route.title,
        maintainState: route.maintainState,
        fullscreenDialog: route.fullscreenDialog);
  }

  static PageRouteBuilder _withTransition(CustomRoute route) {
    assert(route.transitions != null);
    return PageRouteBuilder(
        pageBuilder: route.pageBuilder ??= (cxt, _, __) => route.builder(cxt),
        settings: route.settings,
        transitionDuration: route.duration,
        transitionsBuilder: route.transitions,
        opaque: route.opaque,
        barrierDismissible: route.barrierDismissible,
        barrierColor: route.barrierColor,
        barrierLabel: route.barrierLabel,
        maintainState: route.maintainState,
        fullscreenDialog: route.fullscreenDialog);
  }

  static final _routes = Set<CustomRoute>();

  static bool add(CustomRoute value) => _routes.add(value);

  static void addAll(Iterable<CustomRoute> elements) =>
      _routes.addAll(elements);

  static const initialRoute = '/';

  static GlobalKey<NavigatorState> navigatorKey =
      GlobalKey<NavigatorState>(debugLabel: 'Router');

  static NavigatorState get navigator => navigatorKey.currentState;

  static BuildContext get context => navigatorKey.currentContext;
}
